package io.provis.assembly.action.artifact;

import io.provis.model.Action;
import io.tesla.proviso.archive.Archiver;

import java.io.File;
import java.io.IOException;

import javax.inject.Named;

import org.sonatype.aether.artifact.Artifact;

import com.google.inject.Inject;

/**
 * The unpack is an operation that results in any number of artifacts and resources being contributed to the runtime. The archive to be unpacked can
 * make the metadata about its contents available, or we need to determine the information about the contents by examining the contents.
 * 
 * @author jvanzyl
 *
 */
@Named("unpack")
public class UnpackAction implements Action {

  //
  // Configuration
  //
  private String includes;
  private String excludes;
  private boolean useRoot;
  private boolean flatten;

  private Artifact artifact;
  private File outputDirectory;
  
  //
  // Components
  //
  private Archiver archiver;

  @Inject
  public UnpackAction(Archiver archiver) {
    this.archiver = archiver;
  }
  
  public void execute() {
    
    if (!outputDirectory.exists()) {
      outputDirectory.mkdirs();
    }

    File archive = artifact.getFile();
    
    try {
      archiver.unarchive(archive, outputDirectory, includes, excludes, useRoot, flatten);
    } catch (IOException e) {
      throw new RuntimeException(e);
    }
  }
}