package io.provis.provision;

import java.io.File;

import javax.inject.Inject;
import javax.inject.Named;
import javax.inject.Provider;

import org.apache.maven.repository.internal.MavenRepositorySystemSession;
import org.apache.maven.settings.Settings;
import org.apache.maven.settings.building.DefaultSettingsBuilderFactory;
import org.apache.maven.settings.building.DefaultSettingsBuildingRequest;
import org.apache.maven.settings.building.SettingsBuilder;
import org.apache.maven.settings.building.SettingsBuildingException;
import org.apache.maven.settings.building.SettingsBuildingRequest;
import org.sonatype.aether.RepositorySystem;
import org.sonatype.aether.RepositorySystemSession;
import org.sonatype.aether.repository.LocalRepository;
import org.sonatype.inject.Nullable;

@Named
public class RepositorySystemSessionProvider implements Provider<RepositorySystemSession> {

  private RepositorySystem repositorySystem;
  private String localRepository;

  @Inject
  public RepositorySystemSessionProvider(RepositorySystem repositorySystem, @Nullable @Named("${localRepository}") String localRepository) {
    this.repositorySystem = repositorySystem;
  }

  public RepositorySystemSession get() {
    final MavenRepositorySystemSession repoSession = new MavenRepositorySystemSession();
    repoSession.setLocalRepositoryManager(repositorySystem.newLocalRepositoryManager(new LocalRepository(getLocalRepository())));
    repoSession.setTransferListener(new AetherTransferListener());
    return repoSession;
  }

  private String getLocalRepository() {

    if (localRepository == null || localRepository.length() <= 0) {
      Settings settings = readMavenSettings();
      localRepository = settings.getLocalRepository();
      if (localRepository == null || localRepository.length() <= 0) {
        localRepository = new File(System.getProperty("user.home"), ".m2/repository").getAbsolutePath();
      }
    }

    return localRepository;
  }

  private Settings readMavenSettings() {
    SettingsBuilder settingsBuilder = new DefaultSettingsBuilderFactory().newInstance();
    SettingsBuildingRequest request = new DefaultSettingsBuildingRequest();
    request.setSystemProperties(System.getProperties());
    request.setUserSettingsFile(new File(System.getProperty("user.home"), ".m2/settings.xml"));

    Settings settings;
    try {
      settings = settingsBuilder.build(request).getEffectiveSettings();
    } catch (SettingsBuildingException e) {
      settings = new Settings();
    }

    return settings;
  }
}
